
_G.hardpoint_to_mtype = {
   ..'Reactor',
   ..'Torpedo',
   ..'Siege',
   ..'Hangar',
   ..'Berth',
   ..'Missile',
   ..'Capacitor',
   ..'BioMount',

  SmallTurret = 'Light', -- light weapons are always turreted.
    ..'Turret',
  Fixed = 'Medium',
  LargeFixed = 'Heavy',
  LargeTurret = 'Heavy',
  HugeTurret = 'Heavy',
  MaskedBeamMounts = 'Heavy',
}

-- force the number of weapons of the given type to be
-- divisible the given integer.
_G.mtype_to_divides = {
  Heavy = 2,
  SuperHeavy = 2,
  Light = 0.5,
  Siege = 5,
}

local class_to_n = newproxy | function(s)
  return 2
end 

local class_to_size = newproxy | function(s)

  if s<=3
    return 5
  end

  if s==4
    return 10
  end

  if s==5
    return 15
  end

  return 20
end

function ShipDrawerConfig( _ENV )

  depends 'ShipDrawerConfig'

  if drawer:match('Colony')
    _ENV.default_n=3
    _ENV.default_space=5
  end

  if drawer:match('AK')
    _ENV.default_n=3
    _ENV.default_space=5
  end

  print(..default_n)

  expected_s = expected_s or round | (layer_info.canvas_size)/pixels_per_grid

  if layer_info.canvas_w ~= layer_info.canvas_h
    expected_rect_h = round | (layer_info.canvas_h)/pixels_per_grid
  end

  icon_outline_r= (layer_info.canvas_size > 1500 and 3) or (layer_info.canvas_size <= 750 and 1) or 2

  function hardpoint_position_function(dx,dy)

    return function(v)

      for i=1,#v,2
        v[i],v[i+1]+=dx,dy

        --if drawer=='BoxCS'
        --  console(v[i],v[i+1])
        --end

        v[i+1]=layer_info.canvas_size-v[i+1]
      end
      return normal_location_functions | v
    end
  end


  --local function from_weapon_template( x,y,w,h, ax,ay,aw,ah )
    
  --	local dx=ax-x
  --	local dy=ay-y

  --end

  if file_exists(layers_dir..'weapon_centers.table')
    
    function weapon_grid_align(v)
      
      local r = copy(v)
      local centers = (read_file_table | layers_dir..'weapon_centers.table').weapons

      for i,id in ipairs(v)

        r[2*i-1], r[2*i] = unpack | centers![id]

        --r[i],v[i+1]+=dx,dy

        --if drawer=='BoxCS'
        --  console(v[i],v[i+1])
        --end

       -- r[2*i]=layer_info.canvas_size-r[2*i]
      end
      return normal_location_functions | r

    end

  end


  alignment_grid = hardpoint_position_function(31.5,31.5)
  box_defense_align = hardpoint_position_function( 31.5 + .5*(13-16), 31.5 + .5*(33-12) )
  box_torpedo_align = hardpoint_position_function( 31.5 - .5*(391-365), 31.5 - .5*(27-108) )
  box_cruiser_turret_align = hardpoint_position_function( 31.5 - .5*(7-13), 31.5 - .5*(181-231) )
  box_old_flak_align = hardpoint_position_function( 31.5 - (378-369), 31.5 - .5*(468-503)+17 )

  raw_import_align = hardpoint_position_function(0,0)
  blade_large_ion_align = hardpoint_position_function(0,12)
  blade_chaser_align = hardpoint_position_function(0,-9)
  blade_ion_align = hardpoint_position_function(0,9)
  blade_huge_ion_align = hardpoint_position_function(1.5,5.5)
  pd_fusion_missile_align = hardpoint_position_function(.5,4)
  --



  function create_design_mounts(t)


    -- implicitly call any functions left inside the design set.
    for i,v in ipairs(t)
      for j=1,10 do
        if type(v)=='function'
          v=v()
          t[i]=v
        end
      end

      if v.mtype=='Reactor'
        has_reactor=true
      end

      local part_id in v
      if v.hardpoint_type=='MaskedBeamMounts'
        --print(..part_id)
      end

      if type(part_id)=='number'
        v.mlocation=part_id
      elseif type(part_id)=='table'
        v.mlocation_list=part_id
      end

      v.part_id=nil
       
      v.i=i
    end


    if not t.no_thrusters
      add_thrusters_slot(t)
    else

    end

  

    return t
  end

  local hull_class = hull_class or expected_s!

  insert(_ENV,..hull_class)

  local default_n = _ENV.default_n or class_to_n[hull_class]
  local default_space = _ENV.default_space or class_to_size![hull_class]

  insert(_ENV,..default_n)
  insert(_ENV,..default_space)


  function add_thrusters_slot(t)
    table_append(t,|) Built_In 'Maneuvering Engine'
    t[#t].i = #t
  end


  function Built_In(part,space)
    local _TRACE_INFO=part
    assert(PartInfo[part])
    return {
      mtype = 'Built-In',
      space = space or default_space,
      forced_part = part!,      
    },
  end

  function Hardpoint(hardpoint_type)
    return function(arg)
      arg=arg?
      local n = arg.n or round(default_n* (arg.space_mult or 1) )

      local mtype = arg?.mtype or hardpoint_to_mtype![hardpoint_type]

      local divides = mtype_to_divides[mtype]

      if divides
        n=max( divides,  round | round(n/divides)*divides )
      end

      local mount = {
        ..mtype,
        ..hardpoint_type,
        ..n,
        space = default_space,
        ..arg.hints,
      }

      if hardpoint_type=='Hangar'
        mount.n= round | mount.n*2
      end

      local part_id in arg
      if type(part_id)=='number'
        mount.mlocation=part_id
      elseif type(part_id)=='table'
        mount.mlocation_list=part_id
      end

      return mount
    end
  end

  function SystemSlot(args)
    return {mtype="System", space = round | args?.space or (args?.space_mult or 1) * default_space, ..args?.hints, ..args?.space_mult  }
  end

  function ShieldSlot(args)
    return {mtype="Shield", space = round | args?.space or (args?.space_mult or 1) * default_space, ..args?.hints, ..args?.space_mult  }
  end
  
  function StationSlot(args)
    return {mtype="Station", space = round | args?.space or 2 * default_space, ..args?.hints, n= args?.n or (round | 2*default_space/6) }
  end


  function Slot(mtype,args)
    return {..mtype, space=round | args?.space or (args?.space_mult or 1) * default_space, n= args?.n or (round | default_space/6), ..args?.hints }
  end

  function ArmorSlot(args)
    return {mtype="Armor", space =  round | args?.space or (args?.space_mult or 1) * default_space, ..args?.hints  }
  end

  function part_locations(hardpoint_type)
    return function(t)
      _ENV[hardpoint_type]= normal_location_functions(t)
    end
  end

  --[=[

  function import_alignments(hardpoint)
    local layer_dir = dir..'layers\\'
    local align = read_file_table(layer_dir..'align.table')
    local theme = get_theme(drawer)
    
    local push, centers = pusher {}

    for part,v ; align

      local part_drawer,info = PartDrawer.find( ..part, ..theme, ..hardpoint)

      if part_drawer

        local part_align = read_file_table | info.layer_dir..'align.table'
        console(t2s | part_align)
        
        for i,info ; v

          local x,y in info
          x+= .5* part_align.cx
          y+= .5* part_align.cy
          push(x,y)

          console(..x,..y)
        end

      end

    end

    _ENV[hardpoint] = normal_location_functions ( centers )

    --console(t2s | centers)


  end

  --]=]


end