_ENV = create_file_env(_ENV)

semantic_to_color = color_table {
  light = 'FFFFFF',
  capacitor = 'FFFFFF',
  heavy = 'ffb200',
  missile = '788cff',
  hangar = '788cff',
  planet = '00e57f',
  system = '00ff00',
  shield = 'aa33aa',
  station = '00ff00',
  medium = 'ffff00',
  siege='eb1c25',
  unused='4963f7',
  armor='52aeff',
  error = 'FF00FF',
}


local function load_subdir(subdir)
  local centers = read_file_table | catdir(dir..subdir)..'centers.table'
  local rval = {}
  for i,img in ipairs | file_list | dir..subdir
    local name = img:sub(1,-5)

    if img:sub(-4,-1)=='.png'
      
      local c = centers![name]
      local cx,cy = c!.x,c!.y
      local img =  catdir(dir..subdir)..img
      local icon = {..img,cx=x,cy=y,}

      icon.draw = gl_matrix_fun | function(x,y)
        --local scale=.5
--        call_with_low_tolerance_hint(
         -- draw_scaled_vimage(img,x-scale*cx,y-scale*cy,scale)

         --glScale2D(scale)
         draw_rimage(img,x-cx,y-cy)
         --draw_vimage(x-cx,y-cy,img)

      end

      rval[name]=icon

    end

    rval.none={draw=_SAFE}
  end
  return rval
end

local small_underlay_shifts= {
  ring=-3,
  square=-2,
  grid=-2,
  pentagon=1,
  diamond=1,
}

local large_underlay_shifts= {
  triangle=-4,
}

local underlay_vcenter_shifts = {
  ring=-3,
  square=-2,
  grid=-2,
  pentagon=1,
  diamond=1,
}

function reload_icons()

  small_overlay = load_subdir 'small_overlay'
  small_underlay = load_subdir 'small_underlay'
      
  large_overlay = load_subdir 'large_overlay'
  large_underlay = load_subdir 'large_underlay'

  combined_small_icon = strong_memoizeNN | function(color,underlay,overlay)
    local overlay_icon = small_overlay![overlay]
    local underlay_icon = small_underlay![underlay]

    local dy = small_underlay_shifts[underlay] or 0

    if type(color)=='string'
      color=semantic_to_color[color]
    end
    local vdy = underlay_vcenter_shifts[underlay] or 0

    return {
      draw = restore_color | function(cx,cy)
        cy += dy


        set_frame_color4 | color()
        cx,cy=round_all(cx,cy)
        cx,cy+=.5,.5
        underlay_icon.draw(cx,cy)
        overlay_icon.draw(cx,cy)
        --glBox | centered_box(cx,cy, 40, 40 )
        return centered_box(cx,cy, 40, 40 )
      end,
      ..color,
      draw_vcentered = restore_color | function(cx,cy)
        
        --glBox | centered_box(cx,cy, 40, 40 )

        set_frame_color4 | color()
        cx,cy=round_all(cx,cy)
        cx,cy+=.5,.5
        underlay_icon.draw(cx,cy)
        overlay_icon.draw(cx,cy)
        --glBox | centered_box(cx,cy, 40, 40 )
        return centered_box(cx,cy, 40, 40 )
      end,


    }
  end

  combined_large_icon = strong_memoizeNN | function(color_key,underlay,overlay)
    local overlay_icon = large_overlay![overlay]
    local underlay_icon = large_underlay![underlay]
    local color = semantic_to_color![color_key]
    local dy = large_underlay_shifts[underlay] or 0
    return {
      draw = restore_color | function(cx,cy)

        set_frame_color4 | color()
        cx,cy=round_all(cx,cy)
        cy+=dy
        underlay_icon.draw(cx,cy)
        overlay_icon.draw(cx,cy)
        --glBox | centered_box(cx,cy, 40, 40 )
        return centered_box(cx,cy, 40, 40 )
      end,
    }
  end

end


local type2mountname = {}

local function set_mountname(name,...)
  for i = 1,select('#',...)
    type2mountname[ select(i,...) ] = name
  end  
end


local type2color = {}

local function set_color(color,...)
  for i = 1,select('#',...)
    type2color[ select(i,...) ] = color
  end
end

set_color(semantic_to_color.system,
  'system','drive','reactor','bulkheads','lab','transporter')

set_color(semantic_to_color.shield,
  'shield')
set_mountname('System',
  'system','drive','bulkheads','lab')
set_mountname('Shield',
  'shield')
set_mountname('Power','reactor')

set_color(semantic_to_color.station,
  'construction','warp_net')

set_mountname('Station',
  'construction','warp_net')

set_color(semantic_to_color.missile,
  "missile",'mirv','torpedo','energy_torpedo','fighter')

set_mountname('Missile',"missile")
set_mountname('Torpedo',"torpedo")
set_mountname('Vindicator',"mirv")
set_mountname('Launcher',"energy_torpedo")
set_mountname('Fighter',"fighter")

set_color(semantic_to_color.armor,'armor')
set_mountname('Armor',"armor")


set_color(white,'capacitor','shuttle','cargo')
set_mountname('Busbar',"capacitor")
set_mountname('Shuttle',"shuttle")
set_mountname('Cargo',"cargo")

set_color(white,'anti_missile','pd_beam','pd_kinetic')
set_mountname('Light','anti_missile','pd_beam','pd_kinetic')

set_color(semantic_to_color.medium,
  'beam','kinetic','blast_door','boarding_module','scanner','shield_regenerator')
set_mountname('Medium',
  'beam','kinetic','blast_door','boarding_module','scanner','shield_regenerator')


set_color(semantic_to_color.heavy,
  'heavy_beam','heavy_kinetic')
set_mountname('Heavy',
  'heavy_beam','heavy_kinetic')


set_color(semantic_to_color.siege,'siege')
set_mountname('Siege','siege')

local type2underlay = make_default_table 'ring' {}

local function set_underlay(underlay,...)
  for i = 1,select('#',...)
    type2underlay[ select(i,...) ] = underlay
  end
end

set_underlay( 'diamond',
  'heavy_beam','heavy_kinetic','fighter','torpedo','mirv','energy_torpedo')
set_underlay( 'pentagon',
  'siege')
set_underlay( 'square',
  'cargo','shuttle','armor')
set_underlay('square','shield')



local type2overlay= make_default_table 'transistor' {}

local function set_overlay(overlay,...)
  for i = 1,select('#',...)
    type2overlay[ select(i,...) ] = overlay
  end
end

set_overlay( 'rounded_rectangle','armor')
set_overlay( 'turret','shield')

set_overlay( 'fighter','fighter','shuttle')

set_overlay( 'cargo','cargo')

set_overlay( 'missile','mirv','torpedo','missile', 'anti_missile')
set_overlay( 'small_power','reactor')

set_overlay( 'heat', 'heavy_beam','beam','pd_beam','siege')

set_overlay( 'kinetic', 'heavy_kinetic','kinetic','pd_kinetic')
set_overlay( 'explosive', 'explosive', 'energy_torpedo')
set_overlay( 'turret_cap','blast_door')

set_overlay( 'fixed','boarding_module','scanner')

set_overlay('capacitor', 'capacitor')

set_underlay('grid', 'construction', 'warp_net')
set_overlay('none', 'construction', 'warp_net')
set_overlay('heat','heat')

function PartInfo_Property.small_part_icon(part_info)
  local type,damage_type in part_info

  return combined_small_icon(
    type2color[type] or pink,
    type2underlay[type],
    type2overlay[damage_type or type]
  )
end

function PartInfo_Property.icon_color(part_info)
  local type in part_info

  return 
    type2color[type] or pink


end

function PartInfo_Property.simplified_mount_name(part_info)
  local type in part_info

  return type2mountname![type]
end

local mtype_to_args = {
  Armor = catch( 'armor', 'box','rounded_rectangle' ),
  Light = catch( 'light', 'triangle','small_turret'),
  Medium = catch( 'medium', 'ring','turret'),
  Fixed = catch('medium','ring','fixed'),
  Turret = catch( 'medium', 'ring','turret'),
  LargeFixed = catch( 'heavy', 'diamond','fixed'),
  LargeTurret = catch( 'heavy', 'diamond','turret'),
  Heavy = catch ('heavy','diamond','turret'),
  System = catch('system','ring','transistor'),
  Shield = catch('shield','box','turret'),
  Planet = catch('planet','hexagon','fixed'),
  SuperHeavy = catch('siege','pentagon','fixed'),
  Siege = catch('siege','pentagon','fixed'),
  Hangar = catch('missile','diamond','fighter'),
  Torpedo = catch('missile','diamond','missile'),
  Missile = catch('missile','ring','missile'),
  Station = catch('station','grid','none'),
  Transport = catch('light','box','cargo'),
  Berth = catch('light','box','fighter'),
  Capacitor = catch('light','ring','capacitor'),
	BioMount = catch('light','ring','fixed'),
}

function get_mount_icon(mount)
	local _TRACE_TABLE=mount
	local mtype in mount
  local args = mtype_to_args[mount.hardpoint_type] or mtype_to_args![mtype]
  if args
    return combined_large_icon | args()
  else
    return combined_large_icon('error','triangle','charged')
  end
end

function get_mount_small_icon(mount)

  local args = mtype_to_args[mount.hardpoint_type] or mtype_to_args![mount.mtype]

  local color,underlay,overlay = args()

  if underlay=='triangle'
    underlay='ring'
  elseif underlay=='box'
    underlay='square'
  end

  if args
    return combined_small_icon(color,underlay,overlay)
  else
    return combined_small_icon('error','triangle','charged')
  end
end


--[=[

-- for use in the tech tree
function get_part_small_icon(part)
  --local args = mtype_to_args[mount.hardpoint_type] or mtype_to_args[mount.mtype]
  local info =PartInfo[part]
  local type,damage_type in info

  if (part=='Assault Shuttles') or (part=='Armor Brigade')
    return combined_small_icon('missile','square','fighter')
  end

  if type=='armor'
    return combined_small_icon('armor', 'square','rounded_rectangle' )
  end

  if type=='shield'
    return combined_small_icon('armor','ring','transistor') 
  end

  if type=='reactor'
    return combined_small_icon('medium','ring','transistor') 
  end

  if not info.is_weapon
    return combined_small_icon('system','ring','transistor')
  end

  if type=='missile'
    return combined_small_icon('missile','diamond','missile')
  elseif type=='anti_missile'
    return combined_small_icon('light','ring','missile')
  elseif type=='torpedo'
    return combined_small_icon('siege','pentagon','missile')
  elseif type=='energy_torpedo'
    return combined_small_icon('heavy','diamond','explosive')
  end

  if type=='fighter'
    return combined_small_icon('missile','diamond','fighter')
  --  return combined_small_icon('missile','diamond','fighter')
  end

  local damage_icon
  if damage_type == 'explosive'
    damage_icon='explosive'
  elseif damage_type=='heat'
   -- damage_icon='heat'
  end

  if type=='heavy_beam'
    return combined_small_icon('heavy', 'diamond',damage_icon or 'charged')
  elseif type=='beam'
    return combined_small_icon('medium', 'diamond',damage_icon or 'charged')
  elseif type=='pd_beam'
    return combined_small_icon('light', 'ring',damage_icon or 'charged')
  elseif type=='siege'
    return combined_small_icon('siege', 'pentagon',damage_icon or 'charged')
  end

  if type=='heavy_kinetic'
    return combined_small_icon('heavy', 'diamond','kinetic')
  elseif type=='kinetic'
    return combined_small_icon('medium', 'diamond','kinetic')
  elseif type=='pd_kinetic'
    return combined_small_icon('light', 'ring','kinetic')
  end

  if type=='heavy_kinetic'
    return combined_small_icon('heavy', 'diamond','kinetic')
  elseif type=='kinetic'
    return combined_small_icon('medium', 'diamond','kinetic')
  elseif type=='pd_kinetic'
    return combined_small_icon('light', 'ring','kinetic')
  end


  return combined_small_icon('error','square','charged')
end

-]=]

reload_icons()